@@ -15,7 +15,7 @@ else:
 
 
     class ndarrayDelegator(object):
-        SKIP_KENSU_FIELDS = ["_ndarray__k_nd", "INTERCEPTORS"]
+        SKIP_KENSU_FIELDS = ["_ndarray__k_nd", "INTERCEPTORS", "_ksu_loc_id"]
         SKIP_KENSU_METHODS = ["get_nd", "kensu_init", "to_string"]
 
 
@@ -26,6 +26,7 @@ else:
             elif name == "__class__":
                 return attr_value
             elif name in ['mean','std']:
+                # fixme: lin lost?
                 attr_value = object.__getattribute__(self.get_nd(), name)
                 return attr_value
             elif name in ['round','reshape']:
@@ -60,8 +61,8 @@ else:
 
                 # set_axis updates the DataFrame directly (self) and returns None, we loose the initial dataframe
                 # so we need to create the Kensu object before
-                orig_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(nd, kensu.default_physical_location_ref))
-                orig_sc = eventually_report_in_mem(kensu.extractors.extract_schema(orig_ds, nd))
+                orig_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(kensu_nd, kensu.default_physical_location_ref))
+                orig_sc = eventually_report_in_mem(kensu.extractors.extract_schema(orig_ds, kensu_nd))
 
                 new_args=remove_ksu_wrappers(args)
                 new_args = tuple(new_args)
@@ -117,8 +118,8 @@ else:
 
 
                     else:
-                        orig_ds = kensu.extractors.extract_data_source(nd, kensu.default_physical_location_ref)._report()
-                        orig_sc = kensu.extractors.extract_schema(orig_ds, nd)._report()
+                        orig_ds = kensu.extractors.extract_data_source(kensu_df, kensu.default_physical_location_ref)._report()
+                        orig_sc = kensu.extractors.extract_schema(orig_ds, kensu_df)._report()
 
                         result_ds = kensu.extractors.extract_data_source(result, kensu.default_physical_location_ref)._report()
                         result_sc = kensu.extractors.extract_schema(result_ds, result)._report()
@@ -139,6 +140,7 @@ else:
 
         def __getitem__(self, item):
             returned = self.get_nd()[item]
+            # fixme: input lineage lost
             if isinstance(returned,str):
                 return returned
             else:
@@ -156,14 +158,14 @@ else:
                 other_np = other
             return other_np
 
-        def wrapped_ndarray_binary_op(self, other, wrapped_fn, op_title=None):
+        def wrapped_ndarray_binary_op(self, other_input, wrapped_fn, op_title=None):
             op_title = op_title or 'Numpy ' + str(wrapped_fn.__name__)
-            nd = self.get_nd()
-            other_np = ndarray.remove_np_wrapper(other)
+            input_kensu_nd = self
+            other_np = ndarray.remove_np_wrapper(other_input)
             result = ndarray.using(wrapped_fn(other_np))
-            numpy_report(nd, result, op_title)
-            if isinstance(other, ndarray):
-                numpy_report(nd, other, op_title)
+            numpy_report(input_kensu_nd, result, op_title)
+            if isinstance(other_input, ndarray):
+                numpy_report(other_input, result, op_title)
             return result
 
         def __sub__(self, other):
@@ -286,10 +288,12 @@ else:
             new_args = tuple(new_args)
 
             result = method(*new_args, **kwargs)
+            kensu_result = ndarray.using(result)
 
-            result_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(result, kensu.default_physical_location_ref,
+            # Note: must pass a kensu wrapped result
+            result_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(kensu_result, kensu.default_physical_location_ref,
                                                            logical_naming=kensu.logical_naming))
-            result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, result))
+            result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, kensu_result))
 
             for element in origins:
                 df = element.get_s()
@@ -304,8 +308,8 @@ else:
 
                             kensu.add_dependencies_mapping(result_sc.to_guid(), str(col), orig_sc.to_guid(), str(col_orig),
                                                          "Numpy Where")
-            result = ndarray.using(result)
-            return result
+
+            return kensu_result
 
         wrapper.__doc__ = method.__doc__
         return wrapper
@@ -321,10 +325,20 @@ else:
             new_args = tuple(new_args)
 
             result = method(*new_args, **kwargs)
-
             original_result = result
-
-            return original_result
+            # with extra kwargs unique sometimes return tuples...
+            if isinstance(original_result, np.ndarray):
+                ksu_result = ndarray.using(original_result)
+                numpy_report(args, ksu_result, "Numpy unique")
+            elif isinstance(original_result, tuple) and all([isinstance(x, np.ndarray) for x in original_result]):
+                ksu_result = tuple([ndarray.using(x) for x in original_result])
+                for ksu_result_item in ksu_result:
+                    numpy_report(args, ksu_result_item, "Numpy unique")
+            else:
+                msg = "Kensu numpy.unique got an unexpected result which may be not fully tracked"
+                logging.warning(msg)
+                ksu_result = original_result
+            return ksu_result
 
         wrapper.__doc__ = method.__doc__
         return wrapper
@@ -336,15 +350,11 @@ else:
             new_args = remove_ksu_wrappers(args)
             new_args = tuple(new_args)
 
-            result = method(*new_args, **kwargs)
+            original_result = method(*new_args, **kwargs)
+            ksu_result = ndarray.using(original_result)
+            numpy_report(args,ksu_result,"Numpy abs")
 
-            original_result = result
-
-            nd = new_args[0]
-
-            numpy_report(nd,result,"Numpy abs")
-
-            return ndarray.using(original_result)
+            return ksu_result
 
         wrapper.__doc__ = method.__doc__
         return wrapper
@@ -356,17 +366,10 @@ else:
             new_args = remove_ksu_wrappers(args)
             new_args = tuple(new_args)
 
-            result = method(*new_args, **kwargs)
-
-            original_result = result
-
-            from kensu.itertools import kensu_list
-
-            nd = new_args[0]
-
-            numpy_report(nd,result,"Numpy abs")
-
-            return ndarray.using(original_result)
+            original_result = method(*new_args, **kwargs)
+            ksu_result = ndarray.using(original_result)
+            numpy_report(args,ksu_result,"Numpy abs")
+            return ksu_result
 
         wrapper.__doc__ = method.__doc__
         return wrapper
@@ -378,14 +381,11 @@ else:
             new_args = remove_ksu_wrappers(args)
             new_args = tuple(new_args)
 
-            result = method(*new_args, **kwargs)
-
-            original_result = result
-
-            for nd in new_args[0]:
-                numpy_report(nd,result,"Numpy concat")
-
-            return ndarray.using(original_result)
+            original_result = method(*new_args, **kwargs)
+            ksu_result = ndarray.using(original_result)
+            for input_nd in args:
+                numpy_report(input_nd,ksu_result,"Numpy concat")
+            return ksu_result
 
         wrapper.__doc__ = method.__doc__
         return wrapper
@@ -395,10 +395,8 @@ else:
 
     def array(method):
         def wrapper(*args, **kwargs):
-
             args = list(args)
             obj = args[0]
-
             if isinstance(obj,list):
                 args[0] = remove_ksu_wrappers(obj)
                 new_args = args
@@ -407,17 +405,29 @@ else:
 
             new_args = tuple(new_args)
 
-            result = method(*new_args, **kwargs)
-
-            original_result = result
-
-            return ndarray.using(result)
+            original_result = method(*new_args, **kwargs)
+            ksu_result = ndarray.using(original_result)
+            numpy_report(obj, ksu_result, "Numpy array")
+            return ksu_result
 
         wrapper.__doc__ = method.__doc__
         return wrapper
 
     array = array(np.array)
 
+    def extract_sc(kensu, nd):
+        try:
+            orig_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(nd, kensu.default_physical_location_ref,
+                                                                                logical_naming=kensu.logical_naming))
+            orig_sc = eventually_report_in_mem(kensu.extractors.extract_schema(orig_ds, nd))
+        except Exception:  # thrown by default extract_data_source/extract_schema
+            msg = "Kensu numpy.array got an unexpected argument which can not be fully tracked (yet): " + str(type(nd))
+            logging.warning(msg)
+            #import traceback
+            #traceback.print_stack()
+            orig_sc = None
+
+        return orig_sc
 
 
     def numpy_report(nd,result,name):
@@ -428,22 +438,26 @@ else:
 
         if isinstance(nd,kensu_list):
             orig_sc_list = nd.deps
-
-
+        elif isinstance(nd, tuple) or isinstance(nd, list):
+            # FIXME: fail tolerance for unexpected arguments
+            for input_nd in nd:
+                if isinstance(input_nd, kensu_list):
+                    for x in input_nd.deps:
+                        orig_sc_list.append(x)
+                else:
+                    orig_sc_list.append(extract_sc(kensu, input_nd))
         else:
-            orig_ds = eventually_report_in_mem(kensu.extractors.extract_data_source(nd, kensu.default_physical_location_ref,
-                                                                                    logical_naming=kensu.logical_naming))
-            orig_sc = eventually_report_in_mem(kensu.extractors.extract_schema(orig_ds, nd))
-            orig_sc_list.append(orig_sc)
+            orig_sc_list.append(extract_sc(kensu, nd))
 
         result_ds = eventually_report_in_mem(
             kensu.extractors.extract_data_source(result, kensu.default_physical_location_ref,
                                                  logical_naming=kensu.logical_naming))
         result_sc = eventually_report_in_mem(kensu.extractors.extract_schema(result_ds, result))
 
-        if kensu.mapping == True:
+        if kensu.mapping:
             for orig_sc in orig_sc_list:
-                for col in [s.name for s in result_sc.pk.fields]:
-                    for col_orig in [s.name for s in orig_sc.pk.fields]:
-                        kensu.add_dependencies_mapping(result_sc.to_guid(), str(col), orig_sc.to_guid(), str(col_orig),
-                                                       name)
\ No newline at end of file
+                if orig_sc is not None:
+                    for col in [s.name for s in result_sc.pk.fields]:
+                        for col_orig in [s.name for s in orig_sc.pk.fields]:
+                            kensu.add_dependencies_mapping(result_sc.to_guid(), str(col), orig_sc.to_guid(), str(col_orig),
+                                                           name)
\ No newline at end of file
