@@ -26,6 +26,9 @@ class ExtractorSupport(object):
     def extract_schema(self, data_source, value):
         pass
 
+    def extract_schema_fields(self, value):
+        pass
+
     def extract_data_source_and_schema(self, value, physical_location):
         pass
 
@@ -51,7 +54,7 @@ class Extractors(object):
     def __init__(self, ):
         self.supports = []
 
-    def add_default_supports(self, pandas_support=True, sklearn_support=True, numpy_support=True, tensorflow_support=False, bigquery_support=False, generic_datasource_info_support=True):
+    def add_default_supports(self, pandas_support=True, sklearn_support=False, numpy_support=True, tensorflow_support=False, bigquery_support=False, generic_datasource_info_support=True, matplotlib_support=False):
         if pandas_support:
             from kensu.pandas.extractor import KensuPandasSupport
             self.add_support(KensuPandasSupport())
@@ -72,6 +75,10 @@ class Extractors(object):
             from kensu.utils.dsl.extractors.generic_datasource_info_support import GenericDatasourceInfoSupport
             self.add_support(GenericDatasourceInfoSupport())
 
+        if matplotlib_support:
+            from kensu.matplotlib.extractor import PlotSupport
+            self.add_support(PlotSupport())
+
 
 
     def add_support(self, support):
@@ -86,7 +93,8 @@ class Extractors(object):
 
     def register_schema(self, ds, schema):
         from kensu.utils.kensu_provider import KensuProvider
-        KensuProvider().instance().register_schema_name(ds=ds, schema=schema)
+        k=KensuProvider().instance()
+        k.register_schema_name(ds=ds, schema=schema)
         return schema
 
     def extract_schema(self, data_source, value):
@@ -96,6 +104,13 @@ class Extractors(object):
 
         raise Exception("Not supported object: " + value.__class__)
 
+    def extract_schema_fields(self, value):
+        for support in self.supports:
+            if support.is_supporting(value):
+                return support.extract_schema_fields(value)
+
+        raise Exception("Not supported object: " + value.__class__)
+
     def extract_data_source_and_schema(self, value, physical_location):
         for support in self.supports:
             if support.is_supporting(value):
