@@ -38,12 +38,15 @@ def eventually_report_in_mem(o):
 
 def get_absolute_path(path):
     import os
+    if path.startswith('gs:/'):
+        return path
     for prefix in ["abfss", "/abfss", "dbfs", "/dbfs"]:
         if path.startswith(prefix):
             path = path.replace(prefix, '')
             prefix = prefix.replace('/', '')
             return prefix + ':' + path
-    return 'file:' + str(os.path.abspath(path))
+    location = 'file:' + str(os.path.abspath(path))
+    return location
 
 
 def maybe_report(o, report):
@@ -84,6 +87,38 @@ def report_all2all_lineage(in_obj, out_obj, op_type, in_inmem=True, out_inmem=Tr
                                                from_guid=in_schema.to_guid(),
                                                from_col=str(col),
                                                type=op_type)
+
+
+def report_simple_copy_with_guessed_schema(input_uri,  # type: str
+                                           output_absolute_uri,  # type: str
+                                           read_schema_from_filename,  # type: str
+                                           operation_type,  # type: str
+                                           ):
+    from kensu.utils.dsl.extractors.external_lineage_dtos import GenericComputedInMemDs
+    try:
+        maybe_schema = None
+        logging.info(f"report_simple_copy_with_guessed_schema input={input_uri} output={output_absolute_uri}")
+        if read_schema_from_filename.endswith('.csv'):
+            try:
+                import kensu.pandas as pd
+                # FIXME: how to handle multiple posibilities for CSV separators? e.g. sep=";"
+                maybe_pandas_df = pd.read_csv(read_schema_from_filename)
+                from kensu.utils.kensu_provider import KensuProvider
+                ksu = KensuProvider().instance()
+                _, schema = extract_ksu_ds_schema(ksu, maybe_pandas_df, report=False, register_orig_data=False)
+                maybe_schema = [(f.name, f.field_type) for f in schema.pk.fields]
+            except Exception as ex:
+                logging.warning("unable to guess schema of .csv() file", ex)
+        GenericComputedInMemDs.report_copy_with_opt_schema(
+            src=input_uri,
+            dest=output_absolute_uri,
+            operation_type=operation_type,
+            maybe_schema=maybe_schema
+        )
+    except Exception as e:
+        logging.warning(f"caught exception in dumb_parse_curl", e)
+
+
 def flatten(d, parent_key='', sep='.'):
     items = []
     if isinstance(d,list):
@@ -110,37 +145,41 @@ def logical_naming_batch(string):
 
 
 def to_datasource(ds_pk, format, location, logical_naming, name):
-        if logical_naming == 'File':
+    if logical_naming == 'File':
+        logical_category = location.split('/')[-1]
+        ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
+
+    elif logical_naming == 'Folder':
+        logical_category = location.split('/')[-2]
+        ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
+
+    elif logical_naming == 'AnteFolder':
+        logical_category = location.split('/')[-3]
+        ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
+
+    elif logical_naming == 'ReplaceNumbers':
+        logical_category = logical_naming_batch(name)
+        ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
+
+    elif callable(logical_naming):
+        # TODO create to_datasource for all extractors - limited to pandas DataFrame and BigQuery for now
+        try:
+            logical_category = logical_naming(location)
+        except Exception as e:
+            logging.warning("data source logical_naming function passed to initKensu or KensuProvider instance"
+                            " returned an exception, "
+                            "using default data source naming convention. \n {}".format(e))
             logical_category = location.split('/')[-1]
-            ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
+        ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
 
-        elif logical_naming == 'Folder':
-            logical_category = location.split('/')[-2]
-            ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
-
-        elif logical_naming == 'AnteFolder':
-            logical_category = location.split('/')[-3]
-            ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
-
-        elif logical_naming == 'ReplaceNumbers':
-            logical_category = logical_naming_batch(name)
-            ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
-
-        elif callable(logical_naming):
-            #TODO create to_datasource for all extractors - limited to pandas DataFrame and BigQuery for now
-            try:
-                logical_category = logical_naming(location)
-            except Exception as e:
-                logging.warning("data source logical_naming function passed to initKensu or KensuProvider instance"
-                                " returned an exception, "
-                                "using default data source naming convention. \n {}".format(e))
-                logical_category = location.split('/')[-1]
-            ds = DataSource(name=name, format=format, categories=['logical::' + logical_category], pk=ds_pk)
-
-        else:
-            ds = DataSource(name=name, format=format, categories=[], pk=ds_pk)
-        return ds
+    else:
+        ds = DataSource(name=name, format=format, categories=[], pk=ds_pk)
+    return ds
 
+def save_stats_json(schemaID,stats):
+    from kensu.utils.kensu_provider import KensuProvider
+    kensu = KensuProvider().instance()
+    kensu.schema_stats[schemaID]=stats
 
 def extract_short_json_schema(result, result_ds):
     fields_set = set()
@@ -162,4 +201,3 @@ def extract_short_json_schema(result, result_ds):
 
     short_result_sc = Schema(name="short-schema:" + result_ds.name, pk=sc_pk)
     return short_result_sc
-
