@@ -99,7 +99,7 @@ class Kensu(object):
         if kensu_host is None:
             kensu_host = kensu_conf.get("api_url")
         if auth_token is None:
-            kensu_auth_token = kensu_conf.get("api_token")
+            kensu_auth_token = os.environ.get('KENSU_API_TOKEN') or kensu_conf.get("api_token")
         else:
             kensu_auth_token = auth_token
 
@@ -171,8 +171,8 @@ class Kensu(object):
         self.api_url = kwargs_or_conf_or_default("api_url", None)
         self.report_to_file = report_to_file
 
-        sdk_pat = kwargs_or_conf_or_default("PAT", None)
-        sdk_url = kwargs_or_conf_or_default("sdk_url", None)
+        sdk_pat = os.environ.get('KENSU_SDK_PAT') or kwargs_or_conf_or_default("PAT", None)
+        sdk_url = os.environ.get('KENSU_SDK_URL') or kwargs_or_conf_or_default("sdk_url", None)
         sdk_verify_ssl = kwargs_or_conf_or_default("sdk_verify_ssl", True)
         if sdk_pat is None:
             self.sdk = sdk.DoNothingSDK()
@@ -213,10 +213,7 @@ class Kensu(object):
     # sets the api url using host if passed, otherwise gets KENSU_API_URL
     def get_kensu_host(self, host=None):
         if host is None:
-            if "KENSU_API_URL" in os.environ:
-                kensu_host = os.environ["KENSU_API_URL"]
-            else:
-                kensu_host = None
+                kensu_host = os.environ.get("KENSU_API_URL")
         else:
             kensu_host = host
 
@@ -707,23 +704,31 @@ class Kensu(object):
                         if context == "DATA_STATS":
                             lineage_ids = set(
                                 [data['data']['links'][i]['lineage']['id'] for i in range(len(data['data']['links']))])
-                            for lineage_id in lineage_ids:
-                                current_rules = self.sdk.get_rules_for_ds_in_project(lds_guid, lineage_id, project_id,
-                                                                                     env_name)
-                                current_range_rules = {i['fieldName']: i['uuid'] for i in
-                                                       current_rules['data']['predicates'] if
-                                                       i['functionName'] == 'Range' and (i['environment'] == env_name)}
-                                current_frequency_rule = [i['uuid'] for i in current_rules['data']['predicates'] if
-                                                          i['functionName'] == 'Frequency' and (
-                                                                      i['environment'] == env_name)]
-                                if fun['name'] == 'Range' and (field_name in current_range_rules):
-                                    self.sdk.update_rule(current_range_rules[field_name], fun)
-                                elif fun['name'] == 'Frequency' and current_frequency_rule:
-                                    self.sdk.update_rule(current_frequency_rule[0], fun)
+                            lineage_id_for_ds={}
+                            for link in data['data']['links']:
+                                for lineage_id in lineage_ids:
+                                    if link['lineage']['id'] == lineage_id:
+                                        lineage_id_for_ds.setdefault(lineage_id,[])
+                                        lineage_id_for_ds[lineage_id] = set(list(lineage_id_for_ds[lineage_id]) + [link['sourceDatasourceId']] + [link['targetDatasourceId']])
 
-                                else:
-                                    self.sdk.create_rule(lds_guid, lineage_id, project_id, process_id, env_name, field_name,
-                                                         fun)
+                            for lineage_id in lineage_id_for_ds:
+                                if lds_guid in lineage_id_for_ds[lineage_id]:
+                                    current_rules = self.sdk.get_rules_for_ds_in_project(lds_guid, lineage_id, project_id,
+                                                                                         env_name)
+                                    current_range_rules = {i['fieldName']: i['uuid'] for i in
+                                                           current_rules['data']['predicates'] if
+                                                           i['functionName'] == 'Range' and (i['environment'] == env_name)}
+                                    current_frequency_rule = [i['uuid'] for i in current_rules['data']['predicates'] if
+                                                              i['functionName'] == 'Frequency' and (
+                                                                          i['environment'] == env_name)]
+                                    if fun['name'] == 'Range' and (field_name in current_range_rules):
+                                        self.sdk.update_rule(current_range_rules[field_name], fun)
+                                    elif fun['name'] == 'Frequency' and current_frequency_rule:
+                                        self.sdk.update_rule(current_frequency_rule[0], fun)
+
+                                    else:
+                                        self.sdk.create_rule(lds_guid, lineage_id, project_id, process_id, env_name, field_name,
+                                                             fun)
 
                         elif context == "LOGICAL_DATA_SOURCE":
                             current_rules = self.sdk.get_all_rules_for_ds(lds_guid)
